use crate::{Instance, Szun};

pub struct Block<'a> {
    pub(crate) instance:&'a mut Instance,
    pub(crate) target:usize,
}
impl<'a> Block<'a> {
    pub fn tag() -> u32 { 0x1e }
    
    fn acquire(&self) -> Option<&Vec<u8>>
    {
        return match self.instance.data.get(self.target) {
            Some(entry) => {
                match &entry.object { Szun::Block(data) => Some(data),
                    _ => None
                }
            }
            None => None
        };
    }

    fn acquire_mut(&mut self) -> Option<&mut Vec<u8>>
    {
        return match self.instance.data.get_mut(self.target) {
            Some(entry) => {
                match &mut entry.object {
                    Szun::Block(data) => Some(data),
                    _ => None
                }
            }
            None => None
        };
    }

    pub fn get(&self) -> Vec<u8>
    {
        return match self.acquire() {
            Some(data) => data.clone(),
            None => vec![],
        }
    }

    pub fn get_ref(&self) -> Option<&Vec<u8>>
    {
        return match self.acquire() {
            Some(data) => Some(data),
            None => None,
        }
    }

    pub fn get_mut(&mut self) -> Option<&mut Vec<u8>>
    {
        return match self.acquire_mut() {
            Some(data) => Some(data),
            None => None,
        }
    }

    pub fn set(&mut self, value:Vec<u8>)
    {
        match self.acquire_mut() {
            Some(data) => { *data = value; }
            None => { }
        }
    }
}
