use crate::{Instance, Szun};

pub struct Boolean<'a> {
    pub(crate) instance:&'a mut Instance,
    pub(crate) target:usize,
}
impl<'a> Boolean<'a> {
    pub fn tag() -> u32 { 0x02 }

    fn acquire(&self) -> Option<&bool>
    {
        return match self.instance.data.get(self.target) {
            Some(entry) => {
                match &entry.object { Szun::Boolean(data) => Some(data),
                    _ => None
                }
            }
            None => None
        };
    }

    fn acquire_mut(&mut self) -> Option<&mut bool>
    {
        return match self.instance.data.get_mut(self.target) {
            Some(entry) => {
                match &mut entry.object {
                    Szun::Boolean(data) => Some(data),
                    _ => None
                }
            }
            None => None
        };
    }

    pub fn get(&self) -> bool
    {
        return match self.acquire() {
            Some(data) => *data,
            None => false,
        }
    }

    pub fn get_mut(&mut self) -> Option<&mut bool>
    {
        return match self.acquire_mut() {
            Some(data) => Some(data),
            None => None,
        }
    }

    pub fn set(&mut self)
    {
        match self.acquire_mut() {
            Some(data) => { *data = true; }
            None => { }
        }
    }

    pub fn unset(&mut self)
    {
        match self.acquire_mut() {
            Some(data) => { *data = false; }
            None => { }
        }
    }

    pub fn toggle(&mut self)
    {
        match self.acquire_mut() {
            Some(data) => { *data = !*data; }
            None => { }
        }
    }
}
