use crate::{Instance, Szun, Handle};

#[derive(Clone)]
pub struct ListData {
    pub class:u32,
    pub data:Vec<Handle>,
}

pub struct List<'a> {
    pub(crate) instance:&'a mut Instance,
    pub(crate) target:usize,
}
impl<'a> List<'a> {
    pub fn tag() -> u32 { 0x22 }
    
    fn acquire(&self) -> Option<&ListData>
    {
        return match self.instance.data.get(self.target) {
            Some(entry) => {
                match &entry.object { Szun::List(data) => Some(data),
                    _ => None
                }
            }
            None => None
        };
    }

    fn acquire_mut(&mut self) -> Option<&mut ListData>
    {
        return match self.instance.data.get_mut(self.target) {
            Some(entry) => {
                match &mut entry.object {
                    Szun::List(data) => Some(data),
                    _ => None
                }
            }
            None => None
        };
    }

    pub fn get(&self) -> Vec<Handle>
    {
        return match self.acquire() {
            Some(data) => data.data.clone(),
            None => vec![],
        }
    }

    pub fn get_ref(&self) -> Option<&Vec<Handle>>
    {
        return match self.acquire() {
            Some(data) => Some(&data.data),
            None => None,
        }
    }

    pub fn get_mut(&mut self) -> Option<&mut Vec<Handle>>
    {
        return match self.acquire_mut() {
            Some(data) => Some(&mut data.data),
            None => None,
        }
    }

    pub fn set(&mut self, data:Vec<Handle>) -> Result<(),()>
    {
        /* return match self.acquire_mut() {
            Some(ld) => {
                let mut is_valid = true;
                for handle in data {
                    match self.instance.data.get(handle) {
                        Some(entry) => {
                            if entry.object.tag() != ld.class && ld.class != 0 { is_valid = false; break; }
                        }
                        None => { is_valid = false; break; }
                    }
                }

                if is_valid {
                    ld.data = data;
                    Ok(())
                } else {
                    Err(())
                }
            }
            None => Err(())
        } */
        Err(())
    }
}
