use crate::{Instance, Szun};

pub struct Natural<'a> {
    pub(crate) instance:&'a mut Instance,
    pub(crate) target:usize,
}
impl<'a> Natural<'a> {
    pub fn tag() -> u32 { 0x10 }

    fn acquire(&self) -> Option<&u64>
    {
        return match self.instance.data.get(self.target) {
            Some(entry) => {
                match &entry.object { Szun::Natural(data) => Some(data),
                    _ => None
                }
            }
            None => None
        };
    }

    fn acquire_mut(&mut self) -> Option<&mut u64>
    {
        return match self.instance.data.get_mut(self.target) {
            Some(entry) => {
                match &mut entry.object {
                    Szun::Natural(data) => Some(data),
                    _ => None
                }
            }
            None => None
        };
    }

    pub fn get(&self) -> u64
    {
        return match self.acquire() {
            Some(data) => *data,
            None => 0,
        }
    }

    pub fn get_mut(&mut self) -> Option<&mut u64>
    {
        return match self.acquire_mut() {
            Some(data) => Some(data),
            None => None,
        }
    }

    pub fn set(&mut self, value:u64)
    {
        match self.acquire_mut() {
            Some(data) => { *data = value; }
            None => { }
        }
    }
}
